# vim:et:ts=3:sw=3

from random import Random
from Globals import DTMLFile, InitializeClass
from Globals import Persistent
from OFS.Folder import Folder


manage_addRandomFolderForm = DTMLFile('addRandomFolder', globals())


class RandomFolder(Persistent, Folder):
   """ RandomFolder based on OFS.Folder.
   """
   meta_type = 'Random Folder'

   _properties = Folder._properties + (
         { 'id':'randomfolder_behaviour', 'type':'string' },
         { 'id':'randomfolder_filter'   , 'type':'lines'  },
         )

   def __freshenIdList(self):
      self._idlst = list(self.objectIds(
         self.getProperty('randomfolder_filter')))

      self._curid = 0
      self.rand.shuffle(self._idlst)

   def setNextId(self, id=None):
      """ setNextId(str) -> None

          Sets which will be the next item used by the RandomFolder.
      """
      if id is None: return
      id = str(id).strip()

      if not (id in self._idlst):
         raise AttributeError, "no such id '%s'" % id

      self._next_id = id

   def getRandomId(self):
      """ getRandomId() -> str

          Returns the 'id' of the next item which the RandomFolder selects,
          and advances to the next one.
      """
      val = getattr(self, '_next_id', None)

      if val is not None:
         self._next_id = None
      else:
         val = self._idlst[ self._curid ]
         self._curid += 1
         if (self._curid >= len(self._idlst)):
            self.__freshenIdList()

      return val

   def getRandomURL(self):
      """ getRandomURL() -> str

          Returns the URL to the next item which the RandomFolder selects,
          and advances to the next one.
      """
      return self.absolute_url() + "/" + self.getRandomId()

   def nextRandomId(self):
      """ nextRandomItem() -> str

          Gets the 'id' of the next item which will be selected by
          getRandomId().
      """
      next = getattr(self, '_next_id', None)
      if next is not None:
         return next
      else:
         return self._idlst[ self._curid ]

   def randomObjectIds(self):
      """ randomObjectIds() -> list

          Obtains a list with all the 'id's of the items the RandomFolder
          holds, in the order in which they will be selected.
      """
      return self._idlst

   def randomObjectValues(self):
      """ randomObjectValues() -> list

          Like randomObjectIds(), but returns the objects themselves instead
          of only their identifiers.
      """
      return map(lambda ident: self[ident], self._idlst)

   def randomObjectURLs(self):
      """ randomObjectURLs() -> list

          Like randomObjectIds(), but returns the URLs of the objects
          instead of only their identifiers.
      """
      url_base = self.absolute_url + "/"
      return map(lambda ident: url_base + ident, self._idlst)


   def __init__(self, id, title=''):
      Folder.__init__(self, id)
      self.rand  = Random()
      self.title = title

      # __freshenIdList() will take care of these two attributes and of the
      # 'next_random_item' property.
      self._idlst   = None
      self._curid   = None
      self._next_id = None

      # Use redirects by default
      self._setProperty('randomfolder_behaviour', 'redirect')
      # Set initial list of filters
      self._setProperty('randomfolder_filter', [
            "File", "ExtFile", "PropertyFile", # File types
            "Image", "ExtImage", "VarImage",   # Image types
            ])
      self.__freshenIdList()

   #
   # The following two methods ensure that the current permutation of
   # elements is updated.
   #
   def _setObject(self, id_, obj, roles=None, user=None, setOwn=1):
      oldlen = len(self._objects)
      Folder._setObject(self, id_, obj, roles, user, setOwn)
      if (oldlen != len(self._objects)):
         self.__freshenIdList()

   def _delObject(self, id_, dp=1):
      Folder._delObject(id_, dp)
      self.__freshenIdList()


   def index_html(self, REQUEST, RESPONSE):
      """ Selects a random item from the folder and returns it.
      """
      if len(self) == 0:
         return 'no items'
      behaviour = self.getProperty('randomfolder_behaviour', 'redirect')

      if (behaviour == 'redirect'):
         # Perform a redirect to the URL of the (randomly choosed) real item.
         RESPONSE.redirect(self.getRandomURL())
      else:
         # Return the (randomly choosed) real item directly.
         RESPONSE.setHeader('Cache-Control', 'no-cache')
         RESPONSE.setHeader('Pragma', 'no-cache')
         RESPONSE.setHeader('Expires', '-1')
         return self[ self.getRandomId() ].index_html(REQUEST, RESPONSE)


def manage_addRandomFolder(ctx, id, title='', REQUEST=None):
   """ Adds a new RandomFolder object with id *id*.
   """
   id = str(id)
   ctx._setObject(id, RandomFolder(id, str(title)))
   if REQUEST is not None:
      return ctx.manage_main(ctx, REQUEST, update_menu=1)


InitializeClass(RandomFolder)

